function main_interface() {

	this.refresh_discovered_hosts = 30; // in seconds
	this.refresh_running_jobs = 3; // in seconds
	this.retro_feed = null;
	this.discovery_config = null;
	this.communities = new Array();
	this.communities_data_source = null;
	this.discovered_hosts = new Array();
	this.filtered_discovered_hosts = new Array();
	this.discovered_hosts_data_source = null;
	this.parsed_context = null;
	this.community_being_edited = null;
	this.community_being_deleted = null;
	this.discovered_hosts_updater = null;
	this.running_jobs_updater = null;
	this.running_jobs_data_source = null;
	this.discovery = null;
	this.search_delay_counter = null;
	this.selected_elements = new Array();

	this.run = function(context) {

		this.dispatch_host_updater();
		this.dispatch_running_jobs_updater();

		this.resize_columns();
		
		this.parsed_context = json_decode(context);
		if (!this.parsed_context)
			return false;

		this.discovery_config = new discovery_config();
		this.discovery_config.auto_discovery = this.parsed_context.auto_discovery_state;
		this.discovery_config.auto_import = this.parsed_context.auto_import_state;

		if (!this.parse_communities())
			return false;

		this.prewarm_interface();
		this.assign_event_listeners();

		return true;
	}

	this.dispatch_running_jobs_updater = function() {

		if (!this.running_jobs_updater) {
			this.running_jobs_data_source = new kendo.data.DataSource({
				data: new Array()
			});
			this.running_jobs_updater = new running_jobs_updater();
			this.running_jobs_updater.set_callback(main_interface,"update_running_jobs");
			window.setInterval(main_interface.dispatch_running_jobs_updater,this.refresh_running_jobs * 1000);
		}

		main_interface.running_jobs_updater.fetch();

	}

	this.update_running_jobs = function(jobs) {
		this.running_jobs_data_source.data(jobs);
	}

	this.dispatch_host_updater = function() {

		if (!this.discovered_hosts_updater) {
			this.discovered_hosts_data_source = new kendo.data.DataSource({
				data: new Array()
			});
			this.discovered_hosts_updater = new discovered_hosts_updater();
			this.discovered_hosts_updater.set_callback(main_interface,"new_discovered_hosts");
			window.setInterval(main_interface.dispatch_host_updater,this.refresh_discovered_hosts * 1000);
		}

		main_interface.discovered_hosts_updater.fetch();
	}

	this.new_discovered_hosts = function(hosts) {
		var i;

		for(i=0; i<hosts.length; i++) {
			this.discovered_hosts.push(hosts[i]);
		}

		this.discovered_hosts_data_source.data(this.discovered_hosts);

		jQuery(".element-snmp-returns").kendoPanelBar({
			expandMode: "single"
		});
		
		return true;
	}

	/**
	 * assign event listeners to buttons,
	 * windows and links
	 */
	this.assign_event_listeners = function() {

		jQuery(window).resize(function() {
			main_interface.resize_columns();
		});

		jQuery("#new-ip").keyup(function() {
			main_interface.verify_input_mask();
		});

		jQuery("#filter-field").keyup(function() {
			if (!main_interface.search_delay_counter)
				main_interface.search_delay_counter = setTimeout(main_interface.search_radix, 300);
		});

		jQuery("#filter-field").keydown(function() {
			var idx;

			if (!main_interface.search_delay_counter)
				return;

			clearTimeout(main_interface.search_delay_counter);
			main_interface.search_delay_counter = null;
		});

		jQuery("#autodiscovery").change(function(event){
			main_interface.update_auto_discovery(event.result);
		});

		jQuery("#autoimport").change(function(event){
			main_interface.update_auto_import(event.result);
		});

		jQuery("#select-all-button").click(function(){
			main_interface.select_all_elements();
		});

		jQuery("#import-button").click(function(){
			main_interface.import_selected();
		});

		jQuery("#remove-button").click(function(){
			main_interface.remove_selected();
		});

		jQuery("a.k-i-funnel-clear").click(function(){
			main_interface.clear_filter(this);
		})

	}

	this.search_radix = function() {

		var val_to_search;
		var filter_result;
		var filtered_ids;
		var filtered_id;
		var host;
		var already_inserted = new Array();
		var i;
		var j;
		var k;

		val_to_search = jQuery("#filter-field").val();
		if (val_to_search == "") {
			main_interface.discovered_hosts_data_source.data(main_interface.discovered_hosts);
			return;
		}


		filter_result = main_interface.discovered_hosts_updater.radix.search(val_to_search, 100000);
		if (filter_result.length < 1) {
			main_interface.discovered_hosts_data_source.data(new Array());
			return;
		}

		main_interface.filtered_discovered_hosts = new Array();

		for(j=0; j<filter_result.length; j++) {


			filtered_ids = filter_result[j];
			for(i=0; i<filtered_ids.length; i++) {

				filtered_id = filtered_ids[i];
				if (already_inserted.indexOf(filtered_id) != -1)
					continue;

				already_inserted.push(filtered_id);
				for(k=0; k<main_interface.discovered_hosts.length; k++) {
					host = main_interface.discovered_hosts[k];
					if (filtered_id == host.discovery_id)
						main_interface.filtered_discovered_hosts.push(host);
				}

			}
		}

		main_interface.discovered_hosts_data_source.data(main_interface.filtered_discovered_hosts);

	}

	this.remove_job = function(id) {
		var job_manager;
		job_manager = new running_jobs_updater();
		job_manager.set_callback(this, "removed_job");
		job_manager.remove_job(id);
	}

	this.removed_job = function(success, obj) {
	}

	this.add_mask_to_discovery = function() {
		var new_mask;

		new_mask = jQuery("#new-ip").val();

		if (!this.discovery)
			this.discovery = new discovery();
		
		this.discovery.add(new_mask, this, "mask_added");
	}

	this.mask_added = function(success, obj) {
		if (!success) {
			_err("main_interface.mask_added: error submiting discovery");
			return;
		}

		jQuery("#new-ip").val("");
		jQuery("#add-ip-button").removeClass("k-state-selected");
		this.running_jobs_updater.fetch();
	}

	this.select_all_elements = function() {
		var elementsView = jQuery("#discovered-elements").data("kendoListView");
		elementsView.select(elementsView.element.children());
	}

	this.toggle_element_details = function(obj) {
		var dataUid = jQuery(obj).attr("data-uid");
		jQuery("*[data-uid = '" + dataUid + "']").children("ul").slideToggle("fast");
		jQuery("*[data-uid = '" + dataUid + "']").children("a").toggleClass("k-i-expand").toggleClass("k-i-collapse");
	}

	this.import_selected = function() {
		var i;
		var host;
		var window_properties = {
			title: "Editing hosts",
			template: "edit_hosts_general",
			hosts_being_edited: new Array(),
			width: 900,
			height: 500,
			center: true,
			on_close: true,
		};

		main_interface.update_selected_elements();

		if (main_interface.selected_elements.length == 0) {
			_err("import with no elements");
			return;
		}

		for(i=0; i<this.selected_elements.length; i++) {
			host = this.selected_elements[i];
			window_properties.hosts_being_edited.push(host.host_name);
		}

		//2be removed
		window_properties.hosts_being_edited.push("loja-bage");

		load_page(window_properties);
	}

	this.remove_selected = function() {

		var updater;

		main_interface.update_selected_elements();

		if (main_interface.selected_elements == 0)
			return;

		updater = new discovered_hosts_updater();
		updater.set_callback(main_interface, "removed_selected");
		updater.remove_discovered_hosts(main_interface.selected_elements);
	}

	this.removed_selected = function(obj) {

		var i;
		var discovered_hosts;
		var deleted_ids;
		var host;
		var aux;

		deleted_ids = new Array();
		for(i=0; i<this.selected_elements.length; i++) {
			deleted_ids.push(this.selected_elements[i].discovery_id);
		}

		discovered_hosts = new Array();
		for(i=0; i<this.discovered_hosts.length; i++) {
			host = this.discovered_hosts[i];
			if (deleted_ids.indexOf(host.discovery_id) == -1)
				discovered_hosts.push(host);
		}
		this.discovered_hosts = discovered_hosts;

		// if we have some hosts filtered
		aux = this.discovered_hosts_data_source.data();
		discovered_hosts = new Array();
		for(i=0; i<aux.length; i++) {
			host = aux[i];
			if (deleted_ids.indexOf(host.discovery_id) == -1)
				discovered_hosts.push(host);

		}
		this.discovered_hosts_data_source.data(discovered_hosts);
		this.update_selected_elements_count();

	}

	this.update_selected_elements = function() {
		var list_view;
		var data;

		list_view = jQuery("#discovered-elements").data("kendoListView");
		data = list_view.dataSource.view(); 

		main_interface.selected_elements = new Array();
		jQuery.map(list_view.select(), function(item) {
			var sel;
			sel = data[jQuery(item).index()]
			main_interface.selected_elements.push(sel);
		});

	}

	this.update_selected_elements_count = function(e) {
		var list_view;
		var selected;

		list_view = jQuery("#discovered-elements").data("kendoListView");
		selected = list_view.select();
		jQuery("#select-info").text(selected.length);
	}

	/** 
	 * apply kendo-ui stuff
	 */
	this.prewarm_interface = function() {


		jQuery(".top-nav ul").kendoMenu();
		jQuery("#autodiscovery").opCheckbox();
		jQuery("#autoimport").opCheckbox();

		jQuery("#discovered-elements").kendoListView({
			dataSource: this.discovered_hosts_data_source,
			selectable: "multiple",
			template: kendo.template(jQuery("#element-template").html()),
			change: main_interface.update_selected_elements_count
		});

		jQuery("#snmp-community-register").kendoListView({
			dataSource: this.communities_data_source,
			selectable: false,
			template: kendo.template(jQuery("#community-template").html())
		});

		jQuery("#discovery-options").kendoPanelBar({
			expandMode: "single",
		});

		
		jQuery("#add-community").click(function() {
			main_interface.toggle_add_community(new snmp_community());
		});

		jQuery("#select-all-button").click(function(){
			main_interface.select_all_elements();
		});

		jQuery("#snmp-form").hide();
		jQuery(".snmp-3").hide();

		jQuery("#ip-list").kendoListView({
			dataSource: this.running_jobs_data_source,
			selectable: false,
			template: kendo.template(jQuery("#ip-template").html())
		});

	}

	this.clear_filter = function(par) {
		jQuery(jQuery(par).prev("input")).val("");
	}

	this.resize_columns = function() {
		if (jQuery(window).width() > 1281) {
			jQuery(".content-right").removeClass("large-8").addClass("large-9");
			jQuery(".content-left").removeClass("large-4").addClass("large-3");
		} else {
			jQuery(".content-right").removeClass("large-9").addClass("large-8");
			jQuery(".content-left").removeClass("large-3").addClass("large-4");
		}
	}

	this.update_auto_discovery = function(to_state) {
		this.discovery_config.auto_discovery = to_state ? 1 : 0;
		this.discovery_config.save_auto_discovery(this,"update_autos_callback");
	}

	this.update_auto_import = function(to_state) {
		this.discovery_config.auto_import = to_state ? 1 : 0;
		this.discovery_config.save_auto_import(this,"update_autos_callback");
	}

	this.update_autos_callback = function(success, obj) {
		if (!success) {
			_err("main_interface.update_autos_callback: error saving auto status");
		}
	}

	this.snmp_version_changed = function() {
		var version = jQuery("#snmp-version").val();
		if (version == "3") {
			jQuery(".snmp-3").show();
			jQuery(".snmp-1").hide();
		} else {
			jQuery(".snmp-1").show();
			jQuery(".snmp-3").hide();
		}
	}
	
	this.confirm_delete_community = function(id) {

		this.community_being_deleted = this.locate_community(id);
		if (!this.community_being_deleted) {
			_err("main_interface.delete_community: invalid community");
			return;
		}

		jQuery.opConfirm( "error",
		                  "medium",
		                  "Confirma",
		                  "Tem certeza que deseja remover esta comunidade?",
		                  main_interface,
		                  "delete_community"
		);

	}

	this.delete_community = function(confirmed) {

		if (!confirmed) {
			this.community_being_deleted = null;
			return;
		}

		this.community_being_deleted.remove(this,"community_deleted");
	}

	this.community_deleted = function(success, comm) {
		this.remove_community_from_list(comm);
		this.communities_data_source.data(this.communities);
	}

	this.toggle_add_community = function(data) {

		var snmp_template;

		snmp_template = kendo.template($("#snmp-template").html());
		jQuery('#snmp-form').html(snmp_template({}));

		jQuery("#snmp-form").toggle();
		jQuery("#snmp-community-register").toggle();
		jQuery("#add-community").toggle();
		jQuery("#add-community + .description-text").toggle();
		jQuery("#snmp-authprotocol").kendoDropDownList();
		jQuery("#snmp-level").kendoDropDownList();
		jQuery("#snmp-privprotocol").kendoDropDownList();

		jQuery("#snmp-version").kendoDropDownList({
			change: main_interface.snmp_version_changed
		});

		this.community_being_edited = kendo.observable(data);
		kendo.bind(jQuery('#snmp-form-fields'), this.community_being_edited);

		jQuery("#save-community").click(function() {
			main_interface.community_being_edited.save(main_interface,"community_saved");
		});

		jQuery("#cancel-community").click(function(){
			jQuery("#snmp-form").toggle();
			jQuery("#snmp-community-register").toggle();
			jQuery("#add-community").toggle();
			jQuery("#add-community + .description-text").toggle();
			main_interface.community_being_edited = null;
		});

		this.snmp_version_changed();

	}

	this.edit_community = function(id) {

		var i;
		var comm;

		comm = this.locate_community(id);
		if (comm != null) {
			this.toggle_add_community(comm);
			return;
		}

		_err("main_interface.edit_community: unable to locate community to edit");
	}

	/**
	 * after save snmp community callback
	 */
	this.community_saved = function(success, comm) {

		if (!success) {
			_err("main_interface.community_saved: unable to save community");
			return;
		}

		if (this.locate_community(comm.community_id) == null)
			this.communities.push(comm);
		else
			this.update_community(comm);


		this.communities_data_source.data(this.communities);

		jQuery("#snmp-form").toggle();
		jQuery("#snmp-community-register").toggle();
		jQuery("#add-community").toggle();
		jQuery("#add-community + .description-text").toggle();

	}

	/**
	 * search by community with `id'
	 * on this.communities
	 */
	this.locate_community = function(id) {

		var i;
		var comm;

		for (i in this.communities) {
			comm = this.communities[i];
			if (comm.community_id == id) {
				return comm;
			}
		}

		return null;
	}

	this.remove_community_from_list = function(c) {

		var i;
		var comm;
		var new_comms = new Array();

		for(i in this.communities) {
			comm = this.communities[i];
			if (c.community_id !=  comm.community_id)
				new_comms.push(comm);
		}

		this.communities = new_comms;

	}

	this.update_community = function(c) {

		var i;
		var comm;
		var new_comms = new Array();

		for(i in this.communities) {
			comm = this.communities[i];
			if (c.community_id ==  comm.community_id)
				new_comms.push(c);
			else
				new_comms.push(comm);
		}

		this.communities = new_comms;

	}

	/**
	 * create snmp_community objects
	 * and populates this.communities
	 */
	this.parse_communities = function() {

		var aux;
		var i;
		var comm;

		aux = this.parsed_context.communities;
		if (!aux) {
			_err("main_interface.parse_communities: unable to locate communities on global context");
			return false;
		}

		this.communities = new Array();
		for(i=0; i<aux.length; i++) {
			comm = new snmp_community();
			comm.process_stdobj(aux[i]);
			this.communities.push(comm);
		}

		this.communities_data_source = new kendo.data.DataSource({
			data: this.communities 
		});

		return true;

	}

	this.verify_input_mask = function() {

		var i;
		var j;
		var aux;
		var is_valid = false;
		var entered_mask = "";
		var mask_pieces = [];
		var piece = "";
		var valid_ips_regexes = new Array( 
						/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d\d?\d?$/,
						/^\d{1,3}\.\d{1,3}\.\d{1,3}\.[1-9{1,3}\*]$/,
						/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}-\d{1,3}\.\d{1,3}\.\d{1,3}\.[0-9]{1,3}$/
		);

		for(i=0; i<valid_ips_regexes.length; i++) {

			if (jQuery("#new-ip").val().match(valid_ips_regexes[i]) ) {
				entered_mask = jQuery("#new-ip").val();
				mask_pieces = entered_mask.split(/[\.-]/);

				if (mask_pieces.length != 4 && mask_pieces.length != 8) 
					break;

				if (mask_pieces.length == 4) {

					if (mask_pieces[0] < 255 && mask_pieces[1] && mask_pieces[2]) {
						if (mask_pieces[3] == "*" || mask_pieces[3] < 255) {
							is_valid = true;
						}
					}

				} else {
					aux = 0;
					for(j=0; j<mask_pieces.length; j++) {
						if (mask_pieces[j] > 254) {
							aux = 1;
							break;
						}
					}
					if (aux == 0)
						is_valid = true;
				}

			}

		}

		jQuery("#add-mask-to-discovery").unbind('click');
		if (is_valid) {
			jQuery("#add-ip-button").addClass("k-state-selected");
			jQuery("#add-mask-to-discovery").click(function(){
				main_interface.add_mask_to_discovery();
			});
		} else {
			jQuery("#add-ip-button").removeClass("k-state-selected");
		}
	}

}
